use std::path::Path;
use std::time::Duration;
use std::time::SystemTime;

use rusqlite::params;
use rusqlite::Connection;

use super::DbGlobalAction;
use super::DbLocalAction;

static DB_INIT_QUERY: &str = include_str!("../../migrations/01_init.sql");

pub(crate) struct DbHandle {
    conn: Connection,
}

impl DbHandle {
    pub(crate) fn new<P: AsRef<Path>>(db_path: P) -> Result<DbHandle, Box<dyn std::error::Error>> {
        let mut conn: Connection = Connection::open(db_path)?;
        let tx = conn.transaction()?;
        tx.execute_batch(DB_INIT_QUERY)?;
        tx.commit()?;
        Ok(DbHandle { conn })
    }

    pub(crate) fn get_pending_global_actions(
        &mut self,
    ) -> Result<Vec<DbGlobalAction>, Box<dyn std::error::Error>> {
        let now = SystemTime::now()
            .duration_since(SystemTime::UNIX_EPOCH)?
            .as_secs();
        let mut stmt = self
            .conn
            .prepare("SELECT name, next_run FROM global_action WHERE next_run <= $1")?;
        let rows = stmt
            .query_map(params![now], |row| {
                Ok(DbGlobalAction {
                    name: row.get(0)?,
                    next_run: row.get(1)?,
                })
            })?
            .collect::<Result<Vec<_>, _>>()?;
        Ok(rows)
    }

    pub(crate) fn get_pending_local_actions(
        &mut self,
    ) -> Result<Vec<DbLocalAction>, Box<dyn std::error::Error>> {
        let now = SystemTime::now()
            .duration_since(SystemTime::UNIX_EPOCH)?
            .as_secs();
        let mut stmt = self.conn.prepare(
            "SELECT jail_name, tree, set_name, next_run FROM local_action WHERE next_run <= $1",
        )?;
        let rows = stmt
            .query_map(params![now], |row| {
                Ok(DbLocalAction {
                    jail_name: row.get(0)?,
                    tree: row.get(1)?,
                    set_name: row.get(2)?,
                    next_run: row.get(3)?,
                })
            })?
            .collect::<Result<Vec<_>, _>>()?;
        Ok(rows)
    }

    pub(crate) fn get_time_until_next_action(
        &mut self,
    ) -> Result<Duration, Box<dyn std::error::Error>> {
        let now = SystemTime::now()
            .duration_since(SystemTime::UNIX_EPOCH)?
            .as_secs();
        todo!()
    }
}
